/***************************************************************************//**
 *   @file   Communication.h
 *   @brief  Header file of Communication Driver.
 *   @author Istvan Csomortani (istvan.csomortani@analog.com)
********************************************************************************
 * Copyright 2013(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*******************************************************************************
 *   SVN Revision: $WCREV$
******************************************************************************/
#ifndef _COMMUNICATION_H_
#define _COMMUNICATION_H_

#include <system.h>
#include "altera_avalon_pio_regs.h"
#include "altera_avalon_spi_regs.h"
#include "altera_avalon_jtag_uart_regs.h"

/****************************************************************************/
/********************* GPIO macros redefinitions ****************************/
/****************************************************************************/

#define SPI0_en         (1<<0)
#define SPI0_inv        (1<<1)
#define SPI0_phase      (1<<2)
#define I2C0_en         (1<<3)
#define SPI1_en         (1<<4)
#define SPI1_inv        (1<<5)
#define SPI1_phase      (1<<6)
#define I2C1_en         (1<<7)

#define SPI_BASE_ADDR   SPI_1_P0_BASE
#define SPI_CFG  (SPI1_en | SPI1_inv)

/* LDAC */
#define GPIO1_PIN_OUT   IOWR_ALTERA_AVALON_PIO_DIRECTION(GPIO_BASE, 0x5A)
#define GPIO1_PIN_LOW   IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASE, 0x02)
#define GPIO1_PIN_HIGH  IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASE, 0x02)

/* RESET */
#define GPIO2_PIN_OUT   IOWR_ALTERA_AVALON_PIO_DIRECTION(GPIO_BASE, 0x5A)
#define GPIO2_PIN_LOW   IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASE, 0x10)
#define GPIO2_PIN_HIGH  IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASE, 0x10)

/* CLEAR */
#define GPIO3_PIN_OUT   IOWR_ALTERA_AVALON_PIO_DIRECTION(GPIO_BASE, 0x5A)
#define GPIO3_PIN_LOW   IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASE, 0x08)
#define GPIO3_PIN_HIGH  IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASE, 0x08)

/* POC */
#define GPIO4_PIN_OUT   IOWR_ALTERA_AVALON_PIO_DIRECTION(GPIO_BASE, 0x5A)
#define GPIO4_PIN_LOW   IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASE, 0x40)
#define GPIO4_PIN_HIGH  IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASE, 0x40)

/*****************************************************************************/
/* Functions Prototypes                                                      */
/*****************************************************************************/

/*< Initializes the SPI communication peripheral. */
unsigned char SPI_Init(unsigned char lsbFirst,
                       unsigned long clockFreq,
                       unsigned char clockPol,
                       unsigned char clockEdg);

/*< Writes multiple bytes to the device. */
unsigned char SPI_Write(unsigned char   slaveDeviceId,
                        unsigned char*  data,
                        unsigned char   bytesNumber);
/*< Reads multiple bytes from the device. */
unsigned char SPI_Read( unsigned char   slaveDeviceId,
                        unsigned char*  data,
                        unsigned char   bytesNumber);

/*< Initializes the UART communication peripheral. */
char UART_Init(unsigned long baudRate);

/*< Read one character from the device. */
void UART_ReadChar(char* data);

/*< Writes one character to the device. */
void UART_WriteChar(char data);

/*< Writes a character string to the device. */
void UART_WriteString(const char* string);

#endif  /*_COMMUNICATION_H_*/
