
#include <stdarg.h>
#include <stdlib.h>
#include "Communication.h"
#include "Console.h"

char *intToStr(long number, char base)
{
    unsigned long posNumber  = 0;
    char          negSign    = 0;
    const char    digits[17] = "0123456789ABCDEF";
    static char   buffer[17] = "                ";
    char*         bufferPtr  = &buffer[16];

    if((number < 0) && (base == 10))
    {
        negSign = 1;
        posNumber = -1 * number;
    }
    else
    {
        posNumber = (unsigned long)number;
    }
    do
    {
        *bufferPtr-- = digits[posNumber % base];
        posNumber /= base;
    }
    while(posNumber != 0);
    if(negSign)
    {
        *bufferPtr-- = '-';
    }
    *bufferPtr++;

    return bufferPtr;
}

void CONSOLE_Print(char* str, ...)
{
    char*         stringPtr;
    char          firstParam  = 0;
    char          secondParam = 0;
    unsigned long xMask       = 0;
    unsigned long dMask       = 0;
    char          chNumber    = 0;
    unsigned long multiplier  = 1;
    char*         strArg;
    long          longArg;
    double        doubleArg;
    va_list       argp;

    va_start(argp, str);
    for(stringPtr = str; *stringPtr != '\0'; stringPtr++)
    {
        if(*stringPtr!='%')
        {
            UART_WriteChar(*stringPtr);
            continue;
        }
        stringPtr++;
        firstParam = 0;
        while((*stringPtr >= 0x30) & (*stringPtr <= 0x39))
        {
            firstParam *= 10;
            firstParam += (*stringPtr - 0x30);
            stringPtr++;
        }
        if(*stringPtr == '.')
        {
            stringPtr++;
            secondParam = 0;
            while((*stringPtr >= 0x30) & (*stringPtr <= 0x39))
            {
                secondParam *= 10;
                secondParam += (*stringPtr - 0x30);
                stringPtr++;
            }
        }
        switch(*stringPtr)
        {
        case 'c':
            longArg = va_arg(argp, long);
            UART_WriteChar((char)longArg);
            break;
        case 's':
            strArg = va_arg(argp, char*);
            UART_WriteString(strArg);
            break;
        case 'd':
            longArg = va_arg(argp, long);
            UART_WriteString(intToStr(longArg, 10));
            break;
        case 'x':
            longArg = va_arg(argp, long);
            xMask = 268435456;
            chNumber = 8;
            while(xMask > longArg)
            {
                xMask /= 16;
                chNumber--;
            }
            while(chNumber < firstParam)
            {
                UART_WriteChar('0');
                chNumber++;
            }
            if(longArg != 0)
            {
                UART_WriteString(intToStr(longArg, 16));
            }
            break;
        case 'f':
            doubleArg = va_arg(argp, double);
            chNumber = secondParam;
            while(chNumber > 0)
            {
                multiplier *= 10;
                chNumber--;
            }
            doubleArg *= multiplier;
            if(doubleArg < 0)
            {
                doubleArg *= -1;
                UART_WriteChar('-');
            }
            longArg = (long)doubleArg;
            UART_WriteString(intToStr((longArg / multiplier), 10));
            UART_WriteChar('.');
            dMask = 1000000000;
            chNumber = 10;
            while(dMask > (long)(longArg % multiplier))
            {
                dMask /= 10;
                chNumber--;
            }
            while(chNumber < secondParam)
            {
                UART_WriteChar('0');
                chNumber++;
            }
            UART_WriteString(intToStr((longArg % multiplier), 10));
            break;
        }
    }
    va_end(argp);
}

void CONSOLE_GetCommand(char* command)
{
    char receivedChar = 0;
    unsigned char charNumber   = 0;

    while((receivedChar != '\n') && (receivedChar != '\r'))
    {
        UART_ReadChar(&receivedChar);
        command[charNumber++] = receivedChar;
    }
}

char CONSOLE_CheckCommands(char*       receivedCmd,
                           const char* expectedCmd,
                           double*     param,
                           char*       paramNo)
{
    char cmdType         = 1;
    unsigned char charIndex       = 0;
    char paramString[10] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
    unsigned char paramIndex      = 0;
    unsigned char index           = 0;
    const char digits[17] = "0123456789ABCDEF";
    unsigned char digitIndex = 0;

    while((expectedCmd[charIndex] != '!') &&
          (expectedCmd[charIndex] != '?') &&
          (expectedCmd[charIndex] != '=') &&
          (cmdType != UNKNOWN_CMD))
    {
        if(expectedCmd[charIndex] != receivedCmd[charIndex])
        {
            cmdType = UNKNOWN_CMD;
        }
        charIndex++;
    }
    if(cmdType != UNKNOWN_CMD)
    {
        if(expectedCmd[charIndex] == '!')
        {
            if(receivedCmd[charIndex] == '!')
            {
                cmdType = DO_CMD;
            }
            else
            {
                cmdType = UNKNOWN_CMD;
            }
        }
        if(expectedCmd[charIndex] == '?')
        {
            if(receivedCmd[charIndex] == '?')
            {
                cmdType = READ_CMD;
            }
            else
            {
                cmdType = UNKNOWN_CMD;
            }
        }
        if(expectedCmd[charIndex] == '=')
        {
            if(receivedCmd[charIndex] == '=')
            {
                cmdType = WRITE_CMD;
            }
            else
            {
                cmdType = UNKNOWN_CMD;
            }
        }
        if((cmdType == WRITE_CMD) || (cmdType == READ_CMD))
        {
            charIndex++;
            while((receivedCmd[charIndex] != '\n') &&
                  (receivedCmd[charIndex] != '\r'))
            {
                if((receivedCmd[charIndex] == 0x20))
                {
                    *param = 0;
                    if((paramString[0] == '0') && (paramString[1] == 'x'))
                    {
                        for(index = 2; index < paramIndex; index++)
                        {
                            for(digitIndex = 0; digitIndex < 16; digitIndex++)
                            {
                                if(paramString[index] == digits[digitIndex])
                                {
                                    *param = *param * 16;
                                    *param = *param + digitIndex;
                                }
                            }
                        }
                    }
                    else
                    {
                        *param = atof((const char*)paramString);
                    }
                    param++;
                    *paramNo += 1;
                    for(paramIndex = 0; paramIndex < 10; paramIndex++)
                    {
                        paramString[paramIndex] = 0;
                    }
                    paramIndex = 0;
                    charIndex++;
                }
                else
                {
                    paramString[paramIndex] = receivedCmd[charIndex];
                    charIndex++;
                    paramIndex++;
                }
            }
            if(paramIndex)
            {
                *param = 0;
                if((paramString[0] == '0') && (paramString[1] == 'x'))
                {
                    for(index = 2; index < paramIndex; index++)
                    {
                        for(digitIndex = 0; digitIndex < 16; digitIndex++)
                        {
                            if(paramString[index] == digits[digitIndex])
                            {
                                *param *= 16;
                                *param += digitIndex;
                            }
                        }
                    }
                }
                else
                {
                    *param = atof((const char*)paramString);
                }
                *paramNo += 1;
            }
        }
    }

    return cmdType;
}
