
#include "Console.h"
#include "AD5755.h"

/* List of available commands */
const char* cmdList[] = { "help?",
                          "register?",
                          "register=",
                          "power?",
                          "power=",
                          "range?",
                          "range=",
                          "voltage?",
                          "voltage=",
                          "current?",
                          "current=" };

/* List of commands' descriptions */
const char* cmdDescription[] = {" - Displays all available commands.",
                                " - Get register value for a specified channel. \r\n\t - Arguments :\t+ Register address : 0x00 .. 0x1A",
                                " - Set register value. \r\n\t - Arguments : \t+ Type : 0 - Data register, 1- Control register\r\n\t\t\t+ Register Address : 0x00, 0x02 .. 0x07 \r\n\t\t\t+ Channel : 0x00 .. 0x03 \r\n\t\t\t+ The desired value",
                                /*-------------------------------------------*/
                                " - Displays the power state of the dc-to-dc converters, DAC and internal amplifiers for the selected channel. \r\n\t - Arguments :\t+ Channel : 0x00 .. 0x03",
                                " - Set the power state of the dc-to-dc converters, DAC and internal amplifiers for the selected channel. \r\n\t - Arguments :\t+ Channel : 0x00 .. 0x03",
                                " - Displays the range of the selected channel. \r\n\t - Arguments :\t+ Channel : 0x00 .. 0x03",
                                " - Set the range of the selected channel. \r\n\t - Arguments :\t+ Channel : 0x00 .. 0x03\r\n\t\t\t+ Range : (0..6) ",
                                " - Displays the output voltage of a specified channel. \r\n\t - Arguments :\t+ Channel : 0x00 .. 0x03",
                                " - Set the output voltage of a specified channel. \r\n\t - Arguments :\t+ Channel : 0x00 .. 0x03, \r\n\t\t\t+ Value of the the desired voltage",
                                " - Displays the output current of a specified channel. \r\n\t - Arguments :\t+ Channel : 0x00 .. 0x03",
                                " - Set the output current of a specified channel. \r\n\t - Arguments :\t+ Channel : 0x00 .. 0x03, \r\n\t\t\t+ Value of the desired voltage"};
const char* cmdExample[] = {"",
                            "To get the data register 0x00's value type: datareg?0x00",
                            "To set 0xFF in the data register 0x01 at channel 0x02 type: register=0x00 0x01 0x02 0xFF",
                            /*-----------------------------------------------*/
                            "To get the power state for channel 0x01 type: power?0x01",
                            "To set the power state for channel 0x01 type: power=0x01",
                            "To get the range parameter for channel 0x01 type:range?0x01",
                            "To set the range parameter to 0v to 10V for channel 0x01 type: range=0x01 0x01"
                            "To get the voltage for the first channel type: voltage?0x00",
                            "To set 0x00FF voltage for the first channel type: voltage=0x00 0x00FF",
                            "To get the current for the first channel type: current?0x00",
                            "To set 0x00FF voltage for the first channel type: current=0x00 0x00FF"};

char cmdNo = (sizeof(cmdList) / sizeof(const char*));

/* help? implementation */
void cmd0(double* param, char paramNo)
{
    unsigned char displayCmd = 0;

    CONSOLE_Print("Available commands:\r\n");
    for(displayCmd = 0; displayCmd < cmdNo; displayCmd++)
    {
        CONSOLE_Print("%s\r\n\t%s", (char*)cmdList[displayCmd], \
                                            (char*)cmdDescription[displayCmd]);
        CONSOLE_Print("\r\n");
    }
}

/* register? implementation - Register ReadBack*/
void cmd1(double* param, char paramNo)
{
    short   regValue       = 0;
    char    registerAddr   = 0;

    if(paramNo >= 1)
    {   /* Validate address*/
        if(param[0] < 0)
        {
            param[0] = 0;
        }
        if(param[0] > 26)
        {
            param[0] = 26;
        }
        /* Address wrapping */
        switch((int)param[0])
        {
            case  0 :
                {
                    registerAddr = AD5755_RD_DATA_REG(0);
                    break;
                }
            case  1 :
                {
                    registerAddr = AD5755_RD_DATA_REG(1);
                    break;
                }
            case  2 :
                {
                    registerAddr = AD5755_RD_DATA_REG(2);
                    break;
                }
            case  3 :
                {
                    registerAddr = AD5755_RD_DATA_REG(3);
                    break;
                }
            case  4 :
                {
                    registerAddr = AD5755_RD_CTRL_REG(0);
                    break;
                }
            case  5 :
                {
                    registerAddr = AD5755_RD_CTRL_REG(1);
                    break;
                }
            case  6 :
                {
                    registerAddr = AD5755_RD_CTRL_REG(2);
                    break;
                }
            case  7 :
                {
                    registerAddr = AD5755_RD_CTRL_REG(3);
                    break;
                }
            case  8 :
                {
                    registerAddr = AD5755_RD_GAIN_REG(0);
                    break;
                }
            case  9 :
                {
                    registerAddr = AD5755_RD_GAIN_REG(1);
                    break;
                }
            case 10 :
                {
                    registerAddr = AD5755_RD_GAIN_REG(2);
                    break;
                }
            case 11 :
                {
                    registerAddr = AD5755_RD_GAIN_REG(3);
                    break;
                }
            case 12 :
                {
                    registerAddr = AD5755_RD_OFFSET_REG(0);
                    break;
                }
            case 13 :
                {
                    registerAddr = AD5755_RD_OFFSET_REG(1);
                    break;
                }
            case 14 :
                {
                    registerAddr = AD5755_RD_OFFSET_REG(2);
                    break;
                }
            case 15 :
                {
                    registerAddr = AD5755_RD_OFFSET_REG(3);
                    break;
                }
            case 16 :
                {
                    registerAddr = AD5755_RD_CODE_REG(0);
                    break;
                }
            case 17 :
                {
                    registerAddr = AD5755_RD_CODE_REG(1);
                    break;
                }
            case 18 :
                {
                    registerAddr = AD5755_RD_CODE_REG(2);
                    break;
                }
            case 19 :
                {
                    registerAddr = AD5755_RD_CODE_REG(3);
                    break;
                }
            case 20 :
                {
                    registerAddr = AD5755_RD_SR_CTRL_REG(0);
                    break;
                }
            case 21 :
                {
                    registerAddr = AD5755_RD_SR_CTRL_REG(1);
                    break;
                }
            case 22 :
                {
                    registerAddr = AD5755_RD_SR_CTRL_REG(2);
                    break;
                }
            case 23 :
                {
                    registerAddr = AD5755_RD_SR_CTRL_REG(3);
                    break;
                }
            case 24 :
                {
                    registerAddr = AD5755_RD_STATUS_REG;
                    break;
                }
            case 25 :
                {
                    registerAddr = AD5755_RD_MAIN_CTRL_REG;
                    break;
                }
            case 26 :
                {
                    registerAddr = AD5755_RD_Dc_DC_CTRL_REG;
                    break;
                }
            default:
            {
                registerAddr = 0;
                break;
            }
        }
        regValue = (short)AD5755_GetRegisterValue(registerAddr);
        if(regValue == -1)
        {
            CONSOLE_Print("AD5755 CRC ERROR!\r\n");
        }
        else
        {
            CONSOLE_Print("register[0x%02x]=0x%02x\r\n", \
                                    (unsigned char)param[0], regValue);
        }
    }
    else
    {
        CONSOLE_Print("Invalid parameter!\r\n");
        CONSOLE_Print("%s%s\r\n", (char*)cmdList[1], (char*)cmdDescription[1]);
        CONSOLE_Print("Example: %s\r\n", (char*)cmdExample[1]);
    }
}

/* register= implementation - Set the value of data registers*/
void cmd2(double* param, char paramNo)
{
    char mode      = 0;
    char address   = 0;
    char channel   = 0;

    if(paramNo >= 4)
    {
        mode    = (char)param[0];
        address = (char)param[1];
        channel = (char)param[2];
        /* Validate mode DATA or CONTROL*/
        if(mode < 0)
        {
            mode = 0;
        }
        if(mode > 1)
        {
            mode = 1;
        }
       /* Validate address*/
        if(mode == 0)
        {
            if( (address < 0) || (address == 1) )
            {
                address = 0;
            }
            if(address > 7)
            {
                address = 7;
            }
        }
        else
        {
            if(address < 0)
            {
                address = 0;
            }
            if(address > 4)
            {
                address = 4;
            }
        }
        /* Validate channel */
        if(channel < 0)
        {
            channel = 0;
        }
        if(channel > 3)
        {
            channel = 3;
        }
        if(mode == 0)
        {
            AD5755_SetRegisterValue(address,
                                    channel,
                                    (unsigned long)param[3]
                                    );
        }
        else
        {
            AD5755_SetControlRegisters( address,
                                        channel,
                                        (unsigned long)param[3]
                                        );
        }
        CONSOLE_Print("Register[0x%02x] at channel 0x%02x is 0x%02x\r\n", \
                                                address, channel, param[3]);
    }
    else
    {
        CONSOLE_Print("Invalid parameters!\r\n");
        CONSOLE_Print("%s%s\r\n", (char*)cmdList[2], (char*)cmdDescription[2]);
        CONSOLE_Print("Example: %s\r\n", (char*)cmdExample[2]);
    }
}

/* power? implementation - Displays the power state of a specified channel */
void cmd3(double* param, char paramNo)
{
    char    channel  = 0;
    short   regValue = 0;
    /* Validate parameters */
    if(paramNo >= 1)
    {
        channel = param[0];
        /* Validate channel */
        if(channel < 0)
        {
            channel = 0;
        }
        if(channel > 3)
        {
            channel = 3;
        }
        regValue = (short)AD5755_GetRegisterValue(AD5755_RD_CTRL_REG((char)channel));
        if(regValue == -1)
        {
            CONSOLE_Print("AD5755 CRC ERROR!\r\n");
        }
        else
        {
            if(regValue & AD5755_DAC_DC_DC)
            {
                CONSOLE_Print("Channel %d is ON\r\n", \
                                            (int)channel, regValue );
            }
            else
            {
                CONSOLE_Print("Channel %d is OFF\r\n", \
                                            (int)channel, regValue );
            }
        }
    }
    else
    {
        CONSOLE_Print("Invalid parameters!\r\n");
        CONSOLE_Print("%s%s\r\n", (char*)cmdList[3], (char*)cmdDescription[3]);
        CONSOLE_Print("Example: %s\r\n", (char*)cmdExample[3]);
    }
}

/* power= implementation - Set the power state of a specified channel */
void cmd4(double* param, char paramNo)
{
    short regValue   = 0;
    char  channel    = 0;
    char  onOff      = 0;
    /* Validate parameters */
    if(paramNo >= 2)
    {
        channel = param[0];
        onOff   = param[1];
        /* Validate channel */
        if(channel < 0)
        {
            channel = 0;
        }
        if(channel > 3)
        {
            channel = 3;
        }
        /* 0/1 = OFF/ON */
        if(onOff < 0)
        {
            onOff = 0;
        }
        if(onOff > 1)
        {
            onOff = 1;
        }
        /* Read the DAC control register */
        regValue = (short)AD5755_GetRegisterValue(AD5755_RD_CTRL_REG((char)channel));
        if(regValue == -1)
        {
            CONSOLE_Print("AD5755 CRC ERROR!\r\n");
        }
        else
        {
            if(onOff == 1)
            {   /* Enable the AD5755_DAC_DC_DC and channel output */
                AD5755_SetControlRegisters( AD5755_CREG_DAC,
                                            channel,
                                            (regValue | AD5755_DAC_INT_ENABLE \
                                                      | AD5755_DAC_DC_DC \
                                                      | AD5755_DAC_OUTEN) );
                CONSOLE_Print("Channel %d is ON\r\n", \
                                (int)channel, (regValue | AD5755_DAC_INT_ENABLE \
                                                        | AD5755_DAC_DC_DC \
                                                        | AD5755_DAC_OUTEN) );
            }
            else
            {   /* Disable the AD5755_DAC_DC_DC and channel output */
                AD5755_SetControlRegisters(AD5755_CREG_DAC,
                                          channel,
                                          (regValue & ~(AD5755_DAC_INT_ENABLE | \
                                                        AD5755_DAC_DC_DC | \
                                                        AD5755_DAC_OUTEN)) );
                CONSOLE_Print("Channel %d is OFF\r\n", \
                                (int)channel, (regValue & ~(AD5755_DAC_INT_ENABLE \
                                                          | AD5755_DAC_DC_DC \
                                                          | AD5755_DAC_OUTEN)));
            }
        }
    }
    else
    {
        CONSOLE_Print("Invalid parameters!\r\n");
        CONSOLE_Print("%s%s\r\n", (char*)cmdList[4], (char*)cmdDescription[4]);
        CONSOLE_Print("Example: %s\r\n", (char*)cmdExample[4]);
    }
}

/* range? command implementation - Displays the range of a specified channel */
void cmd5(double* param, char paramNo)
{
    short regValue  = 0;
    short range     = 0;
    char  channel   = 0;
    if(paramNo >= 1)
    {
        channel = param[0];
        /* Validate channel */
        if(channel < 0)
        {
            channel = 0;
        }
        if(channel > 3)
        {
            channel = 3;
        }
        regValue = (short)AD5755_GetRegisterValue(AD5755_RD_CTRL_REG((char)channel));
        if(regValue == -1)
        {
            CONSOLE_Print("AD5755 CRC ERROR!\r\n");
        }
        else
        {
            range = regValue & 0x07;
            switch(range)
            {
                case 0:
                {
                    CONSOLE_Print("Channel %d range : %d - 0V to 5V\r\n", \
                                            (int)channel, AD5755_R_0_5_V);
                    break;
                }
                case 1:
                {
                    CONSOLE_Print("Channel %d range : %d - 0V to 10V\r\n", \
                                            (int)channel, AD5755_R_0_10_V);
                    break;
                }
                case 2:
                {
                    CONSOLE_Print("Channel %d range : %d - +/-5V\r\n", \
                                            (int)channel, AD5755_R_M5_P5_V);
                    break;
                }
                case 3:
                {
                    CONSOLE_Print("Channel %d range : %d - +/-10V\r\n", \
                                            (int)channel, AD5755_R_M10_P10_V);
                    break;
                }
                case 4:
                {
                    CONSOLE_Print("Channel %d range : %d - 4mA to 20mA\r\n", \
                                            (int)channel, AD5755_R_4_20_MA);
                    break;
                }
                case 5:
                {
                    CONSOLE_Print("Channel %d range : %d - 0mA to 20mA\r\n", \
                                            (int)channel, AD5755_R_0_20_MA);
                    break;
                }
                case 6:
                {
                    CONSOLE_Print("Channel %d range : %d - 0mA to 24mA\r\n", \
                                            (int)channel, AD5755_R_0_24_MA);
                    break;
                }
                default:
                {
                    CONSOLE_Print("Invalid range!\r\n");
                    break;
                }
            }
        }
    }
    else
    {
        CONSOLE_Print("Invalid parameters!\r\n");
        CONSOLE_Print("%s%s\r\n", (char*)cmdList[5], (char*)cmdDescription[5]);
        CONSOLE_Print("Example: %s\r\n", (char*)cmdExample[5]);
    }
}

/* range= command implementation - Displays the range of a specified channel */
void cmd6(double* param, char paramNo)
{
    short   regValue    = 0;
    char    channel     = 0;
    short    range       = 0;

    if(paramNo >= 2)
    {
        channel = (char)param[0];
        range   = (char)param[1];
        /* Validate channel */
        if(channel < 0)
        {
            channel = 0;
        }
        if(channel > 3)
        {
            channel = 3;
        }
        /* Validate range */
        if(range < 0)
        {
            range = 0;
        }
        if(range > 6)
        {
            range = 6;
        }
        regValue = (short)AD5755_GetRegisterValue(AD5755_RD_CTRL_REG((char)channel));
        if(regValue == -1)
        {
            CONSOLE_Print("AD5755 CRC ERROR!\r\n");
        }
        else
        {   // Reset the DAC_R bits
            regValue &= 0xFFFFFFF8;
            AD5755_SetControlRegisters( AD5755_CREG_DAC,
                                        channel,
                                        (regValue | range));
            CONSOLE_Print("The range of channel %d was changed\r\n", channel, (regValue | range));
        }
    }
    else
    {
        CONSOLE_Print("Invalid parameters!\r\n");
        CONSOLE_Print("%s%s\r\n", (char*)cmdList[6], (char*)cmdDescription[6]);
        CONSOLE_Print("Example: %s\r\n", (char*)cmdExample[6]);
    }
}

/* voltage? implementation - Display the value of voltage*/
void cmd7(double* param, char paramNo)
{
    short           cntrValue   = -1;
    unsigned long   regValue    =  0;
    char            channel     =  0;

    if(paramNo >= 1)
    {
        channel = param[0];
        /* Validate channel */
        if(channel < 0)
        {
            channel = 0;
        }
        if(channel > 3)
        {
            channel = 3;
        }
        cntrValue = (short)AD5755_GetRegisterValue(AD5755_RD_CTRL_REG((char)channel));
        if( ((cntrValue & AD5755_DAC_INT_ENABLE) != 0) && \
            ((cntrValue & AD5755_DAC_OUTEN) != 0) && \
            ((cntrValue & AD5755_DAC_DC_DC) != 0) && \
            ((cntrValue & AD5755_DAC_R(2)) == 0) )
        {
            regValue = AD5755_GetRegisterValue(AD5755_RD_DATA_REG((char)channel));
            if(regValue == -1)
            {
                CONSOLE_Print("AD5755 CRC ERROR!\r\n");
            }
            else
            {
                CONSOLE_Print("Channel %d - %1.2f\r\n", (int)channel, regValue);
            }
        }
        else
        {
            CONSOLE_Print("Can not perform the command, incompatible configuration.\r\nCheck out if the channel is enabled, or the output is set up to voltage mode.\r\n");
        }
    }
    else
    {
        CONSOLE_Print("Invalid parameters!\r\n");
        CONSOLE_Print("%s%s\r\n", (char*)cmdList[7], (char*)cmdDescription[7]);
        CONSOLE_Print("Example: %s\r\n", (char*)cmdExample[7]);
    }
}

/* voltage= implementation - Set the value of voltage */
void cmd8(double* param, char paramNo)
{
    short   cntrValue   = -1;
    short   lowerLimit  =  0;
    short   upperLimit  =  0;
    char    channel     =  0;
    float   voltage     =  0;

    if(paramNo >= 2)
    {
        channel = param[0];
        voltage = param[1];
        /* Validate channel */
        if(channel < 0)
        {
            channel = 0;
        }
        if(channel > 3)
        {
            channel = 3;
        }
        /* Validate data */
        cntrValue = (short)AD5755_GetRegisterValue(AD5755_RD_CTRL_REG((char)channel));
        if(cntrValue == -1)
        {
            CONSOLE_Print("AD5755 CRC ERROR!\r\n");
        }
        else
        {
            if( ((cntrValue & AD5755_DAC_INT_ENABLE) != 0) && \
                ((cntrValue & AD5755_DAC_OUTEN) != 0) && \
                ((cntrValue & AD5755_DAC_DC_DC) != 0) && \
                ((cntrValue & AD5755_DAC_R(2)) == 0) )
            {
                lowerLimit = ((cntrValue & AD5755_DAC_R(1)) == 0) ? \
                                0 : ((cntrValue & AD5755_DAC_R(0)) == 0) ? -5 : -10;
                upperLimit = ((cntrValue & AD5755_DAC_R(1)) == 0) ? 5 : 10;
                if(voltage < (float)lowerLimit)
                {
                    voltage = lowerLimit;
                }
                if(voltage > (float)upperLimit)
                {
                    voltage = upperLimit;
                }
            }
            else
            {
                CONSOLE_Print("Can not perform the command, incompatible configuration.\r\nCheck out if the channel is enabled, or the output is set up to voltage mode.\r\n");
            }
            /* Write data to register */
            AD5755_SetVoltage(channel, voltage);
            CONSOLE_Print("Output channel %d was updated to %1.2f\r\n", channel, voltage);
        }
    }
    else
    {
        CONSOLE_Print("Invalid parameters!\r\n");
        CONSOLE_Print("%s%s\r\n", (char*)cmdList[8], (char*)cmdDescription[8]);
        CONSOLE_Print("Example: %s\r\n", (char*)cmdExample[8]);
    }
}

/* current? implementation - Display the value of current */
void cmd9(double* param, char paramNo)
{
    short   cntrValue   = -1;
    short   regValue    =  0;
    char    channel = 0;

    if(paramNo >= 1)
    {
        channel = param[0];
        /* Validate channel */
        if(channel < 0)
        {
            channel = 0;
        }
        if(channel > 3)
        {
            channel = 3;
        }
        cntrValue = (short)AD5755_GetRegisterValue(AD5755_RD_CTRL_REG((char)channel));
        if(cntrValue == -1)
        {
            CONSOLE_Print("AD5755 CRC ERROR!\r\n");
        }
        else
        {
            if( ((cntrValue & AD5755_DAC_INT_ENABLE) != 0) && \
                ((cntrValue & AD5755_DAC_OUTEN) != 0) && \
                ((cntrValue & AD5755_DAC_DC_DC) != 0) && \
                ((cntrValue & AD5755_DAC_R(2)) == 1) )
            {
                regValue = AD5755_GetRegisterValue(AD5755_RD_DATA_REG((char)channel));
                if(regValue == -1)
                {
                    CONSOLE_Print("AD5755 CRC ERROR!\r\n");
                }
                else
                {
                    CONSOLE_Print("Channel %d - %dmA\r\n", (int)channel, regValue);
                }
            }
            else
            {
                CONSOLE_Print("Can not perform the command, incompatible configuration.\r\nCheck out if the channel is enabled, or the output is set up to current mode.\r\n");
            }
        }
    }
    else
    {
        CONSOLE_Print("Invalid parameters!\r\n");
        CONSOLE_Print("%s%s\r\n", (char*)cmdList[9], (char*)cmdDescription[9]);
        CONSOLE_Print("Example: %s\r\n", (char*)cmdExample[9]);
    }
}

/* current= implementation - Set the value of current*/
void cmd10(double* param, char paramNo)
{
    short   cntrValue   = -1;
    short   lowerLimit  =  0;
    short   upperLimit  =  0;
    char    channel     =  0;
    char    current     =  0;
    if(paramNo >= 2)
    {
        channel = param[0];
        current = param[1];
        if(channel < 0)
        {
            channel = 0;
        }
        if(channel > 3)
        {
            channel = 3;
        }

        cntrValue = (short)AD5755_GetRegisterValue(AD5755_RD_CTRL_REG((char)channel));
        if(cntrValue == -1)
        {
            CONSOLE_Print("AD5755 CRC ERROR!\r\n");
        }
        else
        {
            if( ((cntrValue & AD5755_DAC_INT_ENABLE) != 0) && \
                ((cntrValue & AD5755_DAC_OUTEN) != 0) && \
                ((cntrValue & AD5755_DAC_DC_DC) != 0) && \
                ((cntrValue & AD5755_DAC_R(2)) == 1) )
            {
                lowerLimit = ( ((cntrValue & AD5755_DAC_R(1)) == 0) &&
                               ((cntrValue & AD5755_DAC_R(0)) == 0) ) ? 4 : 0;
                upperLimit = ( ((cntrValue & AD5755_DAC_R(1)) == 1) &&
                               ((cntrValue & AD5755_DAC_R(0)) == 0) ) ? 24 : 20;
                if(current < lowerLimit)
                {
                    current = lowerLimit;
                }
                if(current > upperLimit)
                {
                    current = upperLimit;
                }
            }
            else
            {
                CONSOLE_Print("Can not perform the command, incompatible configuration.\r\nCheck out if the channel is enabled, or the output is set up to current mode.\r\n");
            }

            AD5755_SetCurrent(channel, current);
        }
    }
    else
    {
        CONSOLE_Print("Invalid parameters!\r\n");
        CONSOLE_Print("%s%s\r\n", (char*)cmdList[10], (char*)cmdDescription[10]);
        CONSOLE_Print("Example: %s\r\n", (char*)cmdExample[10]);
    }
}

cmdFunction cmdFunctions[11] = {cmd0, cmd1, cmd2, cmd3, cmd4, cmd5, \
                                cmd6, cmd7, cmd8, cmd9, cmd10};

int main(void)
{
    char receivedCmd[20];
    unsigned char cmd;
    double param[5];
    char paramNo;
    char cmdType;
    char invalidCmd;

    /************** Initialization phase of the AD5755 converter *************/
    if(AD5755_Init() == 0x01)
    {
        CONSOLE_Print("AD5755 OK\r\n");
    }
    else
    {
        CONSOLE_Print("AD5755 Err\r\n");
    }

    /* Set the Gain for channel A. */
    AD5755_SetRegisterValue(AD5755_DREG_WR_GAIN,
                            AD5755_DAC_A,
                            0xFFFF);

    /* Set the Offset for channel A. */
    AD5755_SetRegisterValue(AD5755_DREG_WR_OFFSET,
                            AD5755_DAC_A,
                            0x8000);

    /* Set the Clear Code for channel A. */
    AD5755_SetRegisterValue(AD5755_DREG_WR_CLR_CODE,
                            AD5755_DAC_A,
                            0x0000);
    /* Select the output range to 0V to 5V. */
    AD5755_SetChannelRange(AD5755_DAC_A,
                           AD5755_R_0_5_V);

    /* Enable only channel one to be cleared when CLEAR bit is asserted. */
    AD5755_ChannelClearEnable(AD5755_DAC_A, 1);

    /* Set the output voltage for channels B, C and D. */
    AD5755_SetVoltage(AD5755_DAC_B, 1.25);
    AD5755_SetVoltage(AD5755_DAC_C, 2.5);
    AD5755_SetVoltage(AD5755_DAC_D, 5.0);
    /********************** End of initialization ****************************/

    while(1)
    {
        CONSOLE_GetCommand(receivedCmd);
        invalidCmd = 0;
        for(cmd = 0; cmd < cmdNo; cmd++)
        {
            paramNo = 0;
            cmdType = CONSOLE_CheckCommands(receivedCmd, cmdList[cmd], \
                                            param, &paramNo);
            if(cmdType == UNKNOWN_CMD)
            {
                invalidCmd++;
            }
            else
            {
                cmdFunctions[cmd](param, paramNo);
            }
        }
        if(invalidCmd == cmdNo)
        {
            CONSOLE_Print("Invalid command!\r\n");
        }
    }

    return 0;
}
